*-- function to retrieve the next ID value from a 
*-- counter field in SQL Server 7.0
*--
FUNCTION GetNextKey
LPARAMETERS tcEntity

*-- define variables
LOCAL lnRetVal, ;
	loConnection, ;
	lcConnectString, ;
	loCommand, ;
	loParameter1, ;
	loParameter2

*-- init variables
lnRetVal = 0

*-- Create a Connection object and 
*-- set properties in the Connection object.
loConnection = CREATEOBJECT( "ADODB.Connection" )

*-- The CursorLocation property of the Connection
*-- object tells the provider where to create the 
*-- cursor - either server-side cursor or 
*-- client-side cursor.
*--
*-- We'll use a value of 2 for a server-side cursor.
loConnection.CursorLocation = 2

*-- The ConnectionString property is used to specify 
*-- a data source by passing a detailed connection 
*-- string containing a series of argument = value 
*-- statements separated by semicolons and is 
*-- identical to the connection string set up in 
*-- an ODBC connector.
*-- 
*-- NOTE: Replace 'Server1' with your server name and
*--       replace 'sa' with your user ID and
*--       fill in your password for login
lcConnectString  = "Provider=SQLOLEDB.1;" + ;
"Data Source=Server1;User ID=sa;Password= "

*-- Open the connection to the SQL Server 7.0 
*-- database using the properties set above.
loConnection.Open( lcConnectString )

*-- Now that we've established a connection to
*-- our SQL Server 7.0 database, we can create
*-- and set properties in the Command object.
loCommand = CREATEOBJECT( "ADODB.Command" )

*-- The CommandType property is used to assist ADO 
*-- in the evaluation of the CommandText property. 
*-- By specifically identifying the type of Command 
*-- object, ADO can optimize the command text before 
*-- sending it to the provider.
*--
*-- We'll use a value of 4 to tell ADO that we're 
*-- sending the name of a stored procedure.
loCommand.CommandType = 4

*-- The CommandText property contains the text of a 
*-- command represented by a Command object. Usually
*-- this will be an SQL statement, but can also be any
*-- other type of command statement recognized by the 
*-- provider, such as a stored procedure call as we 
*-- use in this example. If the CommandText property 
*-- contains an SQL statement, it must be of the 
*-- particular dialect or version supported by the 
*-- provider's query processor.
*-- 
*-- We'll use 'sp_GetNextKey' as the name of our
*-- stored procedure in SQL Server 7.0
loCommand.CommandText = "sp_GetNextKey"

*-- The ActiveConnection property of the Command 
*-- object is used to indicate the Connection object
*-- over which the specified Command object will 
*-- execute.
*-- We'll use 'loConnection' our connection opened above.
loCommand.ActiveConnection = loConnection

*-- Now create and append two Parameter objects to the
*-- Command object.
*-- NOTE - the type, size, direction and order must 
*-- match the parameters in the stored procedure.
*--
*-- The syntax for creating a parameter object is:
*-- CreateParameter(Name, Type, Direction, Size, Value)
*--
*-- The CreateParameter method is used to create a new
*-- Parameter object with a specified name, type, 
*-- direction, size, and value. Any values passed in 
*-- the arguments are written to the corresponding 
*-- Parameter properties.
*--
*-- All the parameters for the CreateParameter method 
*-- are optional. We'll create our Parameter objects 
*-- and then set the properties individually rather 
*-- than pass the property values in as part of the 
*-- CreateParameter method call.
*--
*-- This method does not automatically append the 
*-- Parameter object to the Parameters collection of 
*-- the Command object. This lets additional properties
*-- be set whose values ADO will validate when the 
*-- Parameter object is appended to the collection.
*-- 
*-- If a variable-length data type is specified in the
*-- Type argument, either a Size argument must be 
*-- passed in or the Size property of the Parameter 
*-- object must be set before appending it to the 
*-- Parameters collection; otherwise, an error occurs.
*--
*-- If a numeric data type (numeric or decimal) is 
*-- specified in the Type argument, then the 
*-- NumericScale and Precision properties must also 
*-- be set.
*-- 
*-- The parameters we use in the CreateParameter method
*-- are as follows:
*-- 
*-- Name - Optional. A string value that contains the 
*--        name of the Parameter object.
*-- 
*-- Type - Optional. An Integer value that specifies 
*--        the data type of the Parameter object.
*--
*-- Direction - Optional. An Integer value that 
*--             specifies the direction of Parameter 
*--             object.
*--
*-- Size - Optional. A Long value that specifies the
*--        maximum length for the parameter value in
*--        characters or bytes.
*--
*-- Value - Optional. A Variant that specifies the 
*--         value for the Parameter object.

*-- create the OUTPUT parameter object
loParameter1 = loCommand.CreateParameter()

*-- set the properties of the OUTPUT parameter
loParameter1.Name = "KeyValue" && name we assign
loParameter1.Type = 3  && integer type parameter
loParameter1.Direction = 2 && output parameter
loParameter1.Size = 4 && maximum of 4 bytes long
loParameter1.Value = 0 && initialize the parameter to 0

*-- append the OUTPUT parameter to the Command object
loCommand.Parameters.Append( loParameter1 )

*-- create the INPUT parameter object
loParameter2 = loCommand.CreateParameter()

*-- set the properties of the INPUT parameter
loParameter2.Name = "KeyName"  && name we assign
loParameter2.Type = 201  && long varchar string type
loParameter2.Direction = 1  && input parameter
loParameter2.Size = 30 && maximum 30 characters
loParameter2.Value = tcEntity && our entity parameter

*-- append the INPUT parameter to the Command object
loCommand.Parameters.Append( loParameter2 )

*-- execute the Command object to call the 
*-- SQL Server 7.0 stored procedure
loCommand.Execute()

*-- This is the tricky part. Although everything in the
*-- help files indicates that the value is returned 
*-- directly, such as a function call would do, the 
*-- result is actually contained in the value property
*--  of the parameter object with the OUTPUT type 
*-- specified ( in our case, the "KeyValue" parameter )
*--
*-- using this as lnRetVal = loCommand.Execute(), 
*-- you will get incorrect or missing results.

*-- retrieve the key value
lnRetVal = loCommand.Parameters("KeyValue").Value

*-- close and release the objects and return the 
*-- value to the caller
RELEASE loCommand
loConnection.Close
RELEASE loConnection
RETURN( lnRetVal )

